import os
from typing import List
import mlflow
import logging
import pandas as pd
import joblib

mlflow.set_tracking_uri(os.environ['MLFLOW_BASE_URL'])
client = mlflow.tracking.MlflowClient(os.environ['MLFLOW_BASE_URL'])
logging.basicConfig(level=logging.WARN)
logger = logging.getLogger(__name__)


def set_exp(exp_name: str) -> None:
    '''
    Set given experiment as active experiment. If experiment does not exist, create an experiment with provided name.
    exp_name – Case sensitive name of an experiment to be activated.
    '''
    return mlflow.set_experiment(exp_name)


class MLClient:
    def __init__(self, exp_name: str):
        
        if not isinstance(exp_name, str):
            raise ValueError(f"exp_name must be string")
        if not exp_name:
            raise ValueError(f"exp_name cannot be null or empty string")
            
        set_exp(exp_name)

        exp_details = mlflow.get_experiment_by_name(exp_name)
        self.name = exp_details.name
        self.location = exp_details.artifact_location
        self.id = exp_details.experiment_id
        self.stage = exp_details.lifecycle_stage
        self.tag = exp_details.tags
        logger.info(f"MLClient successfully instantiated")
        
        
    def search_runs(self,exp_id:str):
        '''
        Search runs and return dataframe of runs.
        '''
        try:
            df = mlflow.search_runs(experiment_ids=exp_id)
            df.columns = df.columns.str.replace('tags.mlflow.runName','run_name')
            exclude_cols = ['tags.mlflow.source.type','tags.mlflow.user', 'tags.mlflow.source.name']
            #column_names =  [x for x not in exclude_cols]
            df = df[df.columns.difference(exclude_cols)]
            return  df   
        except:
            print("Experiment id {} does not exists".format(exp_id))

    def stagging(self, model_name: str,run_id:str,stage:str) -> str:
        '''
        register the model in model registry and transit it to given stage.
        Staging, Production or Archived.
        '''
<<<<<<< HEAD
        result = mlflow.register_model("runs:/" + self.id, model_name)
        staging = client.transition_model_version_stage(
            name=registered_model.loc['name'][0],
            version=registered_model.loc['version'][0],
            stage=stage,
        )

    def change_stage(
        model_name: str,
        ver_list: List[str],
        stage: str
    ) -> None:
        '''
        change stage of model.(Staging, Production or Archived.)
        '''

        list(map(lambda x: client.transition_model_version_stage(
            name=model_name, version=x, stage=stage,), ver_list))
#         staging = client.transition_model_version_stage(
#         name=model_name,
#         version=i,
#         stage=stage,
#         )
#         print('{0} verison {1} archived'.format(model_name,i))

    def model_versions(
        model_name: str
    ) -> None:
        '''
        return model versions if match with filter string.
        '''
        filter_string = "name='{}'".format(model_name)
        results = client.search_model_versions(filter_string)
        #fun = lambda res : print("name={}; run_id={}; version={}".format(res.name, res.run_id, res.version))
        # fun(results)
        for res in results:
            print("name={}; run_id={}; version={}".format(
                res.name, res.run_id, res.version))

    def log_data(
            prun_id: str,
            data_obj,
            location: str) -> None:
        '''
        store joblib objects in model artifact.
        '''
        joblib.dump(data_obj, location)
        with mlflow.start_run(run_id=prun_id):
            mlflow.log_artifact(location, artifact_path='model/'+location+'/')
        #print('data saved successfully....')

    def search_runs(exp_id: str):
        '''
        Search runs and return dataframe of runs.
        '''
        df = mlflow.search_runs(experiment_ids=exp_id)
        df.columns = df.columns.str.replace('tags.mlflow.runName', 'run_name')
        exclude_cols = ['tags.mlflow.source.type',
                        'tags.mlflow.user', 'tags.mlflow.source.name']
        # column_names =  [x for x not in exclude_cols]
        df = df[df.columns.difference(exclude_cols)]
        return df

    def previous_version(
        model_name: str
    ):
        '''
        return list of all previous versions.
        '''

        all_versions = []
        filter_string = "name='{}'".format(model_name)
        for x in client.search_model_versions(filter_string):
            all_versions.append(int(dict(x)['version']))
        all_versions.sort()
        previous_versions = list(map(lambda x: str(x), all_versions))[:-1]
        return previous_versions

    def delete_model_version(
        model_name: str,
        ver_list: List[str]
    ):
        '''
        delete model versions.
        '''
        list(map(lambda version: client.delete_model_version(
            name=model_name, version=version), ver_list))

    def delete_reg_model(
        model_name: str
    ):
=======
        try:
            result = mlflow.register_model("runs:/" + run_id,model_name)
            staging = client.transition_model_version_stage(
                                                            name=result.name,
                                                            version=result.version,
                                                            stage=stage,
                                                            )
            return staging.to_proto()
        except:
            print("Could not register the model {} with run_id {}".format(model_name,run_id))
            
    def change_stage(self,
                 model_name:str,
                 ver_list:List[str],
                 stage:str
                ) -> None:
        '''
        change stage of model.(Staging, Production or Archived.)
        '''
        try:
            list(map(lambda x: client.transition_model_version_stage(name=model_name,version=x,stage=stage,), ver_list))
        except:
            print("Could not change the stage of model {} for versions {} ".format(model_name,ver_list))


    def model_versions(self,
                   model_name:str
                   )-> List:
        '''
        return model versions if match with filter string.
        '''
        try:
            
            filter_string = "name='{}'".format(model_name)
            results = client.search_model_versions(filter_string)
            return list(map(lambda x:x.version, results))
        except:
            print("Could not get the versions of model {}. Try another name ".format(model_name))
            
    def log_data(self,
                 prun_id:str,
                 data_obj,
                 location:str) ->None:
        '''
        store joblib objects in model artifact.
        '''
        joblib.dump(data_obj,location)
        with mlflow.start_run(run_id = prun_id):
            mlflow.log_artifact(location,artifact_path='model/'+location+'/')
        #print('data saved successfully....')            
         
    
    def delete_model_version(self,
                            model_name:str,
                            ver_list:List[str]
                            )->None:
        '''
        delete model versions.
        '''    
        try:
            list(map(lambda version: client.delete_model_version(name=model_name, version=version),ver_list))
        except:
            print("Could not delete the respective versions {} of model {}. Make sure the versions or model exists".format(ver_list,model_name))

    def delete_reg_model(self,
                        model_name:str
                        ):
>>>>>>> 0f45f8721f754b3cc2a7ac809e435277bee854bd
        '''
        delete registered model with all its version.
        '''
        try:
            return client.delete_registered_model(name=model_name)
        except:
            print("Could not delete the registered model {}. Make sure the model is registered".format(model_name))

<<<<<<< HEAD
    def delete_exp(
        experiment_id: str
    ):
        '''
        delete registered model with all its version.
        '''
        client.delete_experiment(experiment_id)

    def delete_run_by_id(run_id: str):
        client.delete_run(run_id)

    def load_model(location):  # (model_name,stage):
        return mlflow.sklearn.load_model(location)
=======
    def delete_exp(self,experiment_id:str):
        '''
        delete registered model with all its version.
        '''
        try:
            
            run_list = self.search_runs(experiment_id)['run_id'].tolist()
            list(map(lambda run: self.delete_run_by_id(run_id=run),run_list))
            return client.delete_experiment(experiment_id)
        except:
            print("Could not delete the experiment with id {}")
            
    def delete_run_by_id(self,run_ids:List):
        try:
            list(map(lambda run: client.delete_run(run_id=run),run_ids))
            return "{} runids successfully deleted".format(run_ids)
        except:
            print("Could not delete run_ids {}. Make sure they exits".format(run_ids))

    def load_model(self,location):
        try:
            return mlflow.sklearn.load_model(location)
        except:
            print("Could  not load the model, Make sure you saved it under the location {}".format(location))
>>>>>>> 0f45f8721f754b3cc2a7ac809e435277bee854bd
