import os
from typing import List
import mlflow
import mlflow.sklearn
from  mlflow.tracking import MlflowClient

from fbprophet import Prophet
from sklearn.metrics import mean_squared_error, r2_score, mean_absolute_error
import numpy as np
import mlflow.pyfunc

mlflow.set_tracking_uri(os.environ['MLFLOW_BASE_URL'])
client = mlflow.tracking.MlflowClient(os.environ['MLFLOW_BASE_URL'])

def run_model(train, test, model_name):   
    with mlflow.start_run(run_name=model_name):
        model=Prophet()
        model.fit(train)

        test_pred = model.predict(test[['ds']])

        mlflow.log_metric("r2_score", r2_score(test['y'],test_pred['yhat']))
        mlflow.log_metric("mean_absolute_error", mean_absolute_error(test['y'],test_pred['yhat']))

        mlflow.log_metric("root_mean_squared_error", np.sqrt(mean_squared_error(test['y'],test_pred['yhat'])))

        mlflow.sklearn.log_model(model, model_name)
        
        return model, test_pred
        
def register_model(run_id, model_name):
    return mlflow.register_model("runs:/" + run_id,model_name)


def load_model(location):#(model_name,stage):
    return mlflow.sklearn.load_model(location)